library IEEE ;
use IEEE.STD_LOGIC_1164.all ;

package	constants is
	type TPicoType is ( pbtI, pbtII, pbt3, pbtS ) ;
	constant PicoType : TPicoType := pbt3 ;
	function ADDRSIZE return natural ;
	function INSTSIZE return natural ;
	function JADDRSIZE return natural ;
	function JDATASIZE return natural ;
end package ;

package body constants is
	function ADDRSIZE return natural is
	begin
		case PicoType is
		when pbtI => return 8 ;
		when pbtII => return 10 ;
		when pbt3 => return 10 ;
		when pbtS => return 10 ;
		end case ;
	end ;
	function INSTSIZE return natural is 
	begin
		case PicoType is
		when pbtI => return 16 ;
		when pbtII => return 18 ;
		when pbt3 => return 18 ;
		when pbtS => return 18 ;
		end case ;
	end ;
	function JADDRSIZE return natural is
	begin
		case PicoType is
		when pbtI => return 9 ;
		when pbtII => return 11 ;
		when pbt3 => return 11 ;
		when pbtS => return 10 ;
		end case ;
	end ;
	function JDATASIZE return natural is
	begin
		case PicoType is
		when pbtI => return 8 ;
		when pbtII => return 9 ;
		when pbt3 => return 9 ;
		when pbtS => return 20 ;
		end case ;
	end ;
end package body ;


library IEEE ;
use IEEE.STD_LOGIC_1164.all ;
use IEEE.STD_LOGIC_ARITH.all ;
use IEEE.STD_LOGIC_UNSIGNED.all ;

library unisim ;
use unisim.vcomponents.all ;

use constants.all;

entity ROM_CODE_SYS is
    port ( 
        clk : in std_logic ;
        reset : out std_logic ;
        address : in std_logic_vector( ADDRSIZE - 1 downto 0 ) ;
        instruction : out std_logic_vector( INSTSIZE - 1 downto 0 )
    ) ;
end entity ROM_CODE_SYS ;

architecture mix of ROM_CODE_SYS is
    component jtag_shifter is
        port ( 
			clk : in std_logic ;
			user1 : out std_logic ;
            write : out std_logic ;
            addr : out std_logic_vector( JADDRSIZE - 1 downto 0 ) ;
            data : out std_logic_vector( JDATASIZE - 1 downto 0 )
        ) ;
    end component ;

    signal jaddr : std_logic_vector( JADDRSIZE - 1 downto 0 ) ;
    signal jdata : std_logic_vector( JDATASIZE - 1 downto 0 ) ;
    signal juser1 : std_logic ;
    signal jwrite : std_logic ;

    attribute INIT_00 : string ;
    attribute INIT_01 : string ;
    attribute INIT_02 : string ;
    attribute INIT_03 : string ;
    attribute INIT_04 : string ;
    attribute INIT_05 : string ;
    attribute INIT_06 : string ;
    attribute INIT_07 : string ;
    attribute INIT_08 : string ;
    attribute INIT_09 : string ;
    attribute INIT_0A : string ;
    attribute INIT_0B : string ;
    attribute INIT_0C : string ;
    attribute INIT_0D : string ;
    attribute INIT_0E : string ;
    attribute INIT_0F : string ;
    attribute INIT_10 : string ;
    attribute INIT_11 : string ;
    attribute INIT_12 : string ;
    attribute INIT_13 : string ;
    attribute INIT_14 : string ;
    attribute INIT_15 : string ;
    attribute INIT_16 : string ;
    attribute INIT_17 : string ;
    attribute INIT_18 : string ;
    attribute INIT_19 : string ;
    attribute INIT_1A : string ;
    attribute INIT_1B : string ;
    attribute INIT_1C : string ;
    attribute INIT_1D : string ;
    attribute INIT_1E : string ;
    attribute INIT_1F : string ;
    attribute INIT_20 : string ;
    attribute INIT_21 : string ;
    attribute INIT_22 : string ;
    attribute INIT_23 : string ;
    attribute INIT_24 : string ;
    attribute INIT_25 : string ;
    attribute INIT_26 : string ;
    attribute INIT_27 : string ;
    attribute INIT_28 : string ;
    attribute INIT_29 : string ;
    attribute INIT_2A : string ;
    attribute INIT_2B : string ;
    attribute INIT_2C : string ;
    attribute INIT_2D : string ;
    attribute INIT_2E : string ;
    attribute INIT_2F : string ;
    attribute INIT_30 : string ;
    attribute INIT_31 : string ;
    attribute INIT_32 : string ;
    attribute INIT_33 : string ;
    attribute INIT_34 : string ;
    attribute INIT_35 : string ;
    attribute INIT_36 : string ;
    attribute INIT_37 : string ;
    attribute INIT_38 : string ;
    attribute INIT_39 : string ;
    attribute INIT_3A : string ;
    attribute INIT_3B : string ;
    attribute INIT_3C : string ;
    attribute INIT_3D : string ;
    attribute INIT_3E : string ;
    attribute INIT_3F : string ;
    attribute INITP_00 : string ;
    attribute INITP_01 : string ;
    attribute INITP_02 : string ;
    attribute INITP_03 : string ;
    attribute INITP_04 : string ;
    attribute INITP_05 : string ;
    attribute INITP_06 : string ;
    attribute INITP_07 : string ;
begin
	I18 : if (PicoType = pbtII) or (PicoType = pbt3) generate
	    attribute INIT_00 of bram : label is "01680160016B0B000C12016801605406CC010147093D0C20016801600160032A" ;
	    attribute INIT_01 of bram : label is "C323C3220300C321037FC31103020160019A016001605412CC010147093D0C20" ;
	    attribute INIT_02 of bram : label is "0F00C3360343C3350340C334C333C3320300C331037FC3260343C3250340C324" ;
	    attribute INIT_03 of bram : label is "50622502025F040103100401031002CBC000032F0160016001CAC001032F02CB" ;
	    attribute INIT_04 of bram : label is "03050401018601CE0236D820A83F021303001280A8C0021303200403028E0501" ;
	    attribute INIT_05 of bram : label is "406101924061018E4061018A4060505E4200505C4240505A428050602680025F" ;
	    attribute INIT_06 of bram : label is "020003001280A8C0020003200403027805015089250202460401031001600196" ;
	    attribute INIT_07 of bram : label is "50854200508342405081428050872680024603050401018601AA0226D820A83F" ;
	    attribute INIT_08 of bram : label is "414650904A0D4A104036C001030001600196408801924088018E4088018A4087" ;
	    attribute INIT_09 of bram : label is "02A402130482038002A402130482034002A402130402030001600186019E01CE" ;
	    attribute INIT_0A of bram : label is "02A402130483034002A402130483032002A402130483030002A40213048203C0" ;
	    attribute INIT_0B of bram : label is "02A402130483038C02A402130483038802A402130483038402A4021304830380" ;
	    attribute INIT_0C of bram : label is "54CB4320830102B4025F030004010160018601B201CE016002A4021304830390" ;
	    attribute INIT_0D of bram : label is "019E01AA0160016054D74320830102B4025F030004070160018601A601CE0160" ;
	    attribute INIT_0E of bram : label is "048203C002A402000482038002A402000482034002A402000402030001600186" ;
	    attribute INIT_0F of bram : label is "0483038002A402000483034002A402000483032002A402000483030002A40200" ;
	    attribute INIT_10 of bram : label is "0483039002A402000483038C02A402000483038802A402000483038402A40200" ;
	    attribute INIT_11 of bram : label is "0410016055194320830102B40246030004010160018601B201AA016002A40200" ;
	    attribute INIT_12 of bram : label is "0160552943808301016002C101DE0300016001860165014C1940017E01A601AA" ;
	    attribute INIT_13 of bram : label is "014C195001DE02C101DE0300016001860165014C1940017E01A601AA04160160" ;
	    attribute INIT_14 of bram : label is "090E090E090E1190A000C910514720044011800101600160553B432083010160" ;
	    attribute INIT_15 of bram : label is "A00001478930415E8937595D490AA90F191001478930415689375955490A090E" ;
	    attribute INIT_16 of bram : label is "CC01014749408B01CB40A00001470909A00001470920A0000147090A0147090D" ;
	    attribute INIT_17 of bram : label is "0B2E0C0BA000016B0B2A0C04A000016B0B1C0C0EA000016B0B120C0AA000556B" ;
	    attribute INIT_18 of bram : label is "0B560C09A000016B0B4D0C09A000016B0B450C08A000016B0B390C0CA000016B" ;
	    attribute INIT_19 of bram : label is "0B900C04A000016B0B740C1CA000016B0B680C0CA000016B0B5F0C09A000016B" ;
	    attribute INIT_1A of bram : label is "0BA10C04A000016B0B9D0C04A000016B0B990C04A000016B0B940C05A000016B" ;
	    attribute INIT_1B of bram : label is "0BB50C04A000016B0BB10C04A000016B0BAD0C04A000016B0BA50C08A000016B" ;
	    attribute INIT_1C of bram : label is "0BCB0C08A000016B0BC60C05A000016B0BBF0C07A000016B0BB90C06A000016B" ;
	    attribute INIT_1D of bram : label is "25C04503A000016B0BFA0C05A000016B0BEA0C10A000016B0BD30C17A000016B" ;
	    attribute INIT_1E of bram : label is "4703A00001DE165001DEA000450155E625804503C5030580C302C404033455DE" ;
	    attribute INIT_1F of bram : label is "A000C7030740C501033455F827404703C7030740C601C302C404033455EF27C0" ;
	    attribute INIT_20 of bram : label is "46224521560BA9C04926A403C426C3255605A9C04926C488A4034205C480A403" ;
	    attribute INIT_21 of bram : label is "A9C04936A403C436C3355618A9C04936C488A4034218C480A403A00048244723" ;
	    attribute INIT_22 of bram : label is "C622C521562BA9C04926C448A403422BC440A403A0004834473346324531561E" ;
	    attribute INIT_23 of bram : label is "C632C531563BA9C04936C448A403423BC440A403A000A403C426C325C824C723" ;
	    attribute INIT_24 of bram : label is "0700060606060606060607001640088A42490884A000A403C436C335C834C733" ;
	    attribute INIT_25 of bram : label is "0884A000462245215659A8C04826C726C6255654A8C04826D780D63007000606" ;
	    attribute INIT_26 of bram : label is "566DA8C04836D780D630070006060700060606060606060607001640088A4262" ;
	    attribute INIT_27 of bram : label is "07060706070608001740094A427B0944A000463245315672A8C04836C736C635" ;
	    attribute INIT_28 of bram : label is "42910944A000C826C725C622C5215686A9C04926D890D7300800070608000706" ;
	    attribute INIT_29 of bram : label is "C531569CA9C04936D890D730080007060800070607060706070608001740094A" ;
	    attribute INIT_2A of bram : label is "014C1960014C1970014C19800165014C1930014C19400168A000C836C735C632" ;
	    attribute INIT_2B of bram : label is "0160014C1950014C19600165014C19300165014C19400168A0000160014C1950" ;
	    attribute INIT_2C of bram : label is "01DE030052D42F010410A000014C19500165014C19300165014C19400147A000" ;
	    attribute INIT_2D of bram : label is "454156FE460101EB03020416030CCF01570B450301DE03004305AFFC530B4503" ;
	    attribute INIT_2E of bram : label is "060D030401EF05840680031B0416CF0256FE45C0A5F056FE460C01EB030356FE" ;
	    attribute INIT_2F of bram : label is "016501C2430B56FA268001EB01EF05400691030001EF05400611030001EF05E1" ;
	    attribute INIT_30 of bram : label is "01DE0314016801AAA000430B0160016001D6016501C2430B0160016001D20147" ;
	    attribute INIT_31 of bram : label is "016801605718433C83010147195001DE032801680160570F4324830101471950" ;
	    attribute INIT_32 of bram : label is "00DDA000033E02F901FA00FAA000016001605721435C83010147195001DE0344" ;
	    attribute INIT_33 of bram : label is "573EC001A000033E020101FE007AA000033E020501FA007CA000033E0264018D" ;
	    attribute INIT_34 of bram : label is "00000000000000000000000000000000000000000000A000573EC201573EC101" ;
	    attribute INIT_35 of bram : label is "0000000000000000000000000000000000000000000000000000000000000000" ;
	    attribute INIT_36 of bram : label is "0000000000000000000000000000000000000000000000000000000000000000" ;
	    attribute INIT_37 of bram : label is "0000000000000000000000000000000000000000000000000000000000000000" ;
	    attribute INIT_38 of bram : label is "0000000000000000000000000000000000000000000000000000000000000000" ;
	    attribute INIT_39 of bram : label is "0000000000000000000000000000000000000000000000000000000000000000" ;
	    attribute INIT_3A of bram : label is "0000000000000000000000000000000000000000000000000000000000000000" ;
	    attribute INIT_3B of bram : label is "0000000000000000000000000000000000000000000000000000000000000000" ;
	    attribute INIT_3C of bram : label is "0000000000000000000000000000000000000000000000000000000000000000" ;
	    attribute INIT_3D of bram : label is "0000000000000000000000000000000000000000000000000000000000000000" ;
	    attribute INIT_3E of bram : label is "0000000000000000000000000000000000000000000000000000000000000000" ;
	    attribute INIT_3F of bram : label is "408C000000000000000000000000000000000000000000000000000000000000" ;
	    attribute INITP_00 of bram : label is "DDDDC3F0C0C3370FFFFF77770FC3030CDC03FFFF222A2222A223FF70FC3F70FF" ;
	    attribute INITP_01 of bram : label is "0F0F0F0F0F0F0F0FFFD7C3FFD7C3FFF0F0F0F0F0F0F0F0F0F0F0F0FFF4F3FFFF" ;
	    attribute INITP_02 of bram : label is "0B0B0B0B71ACB2CCB7743776A8AD3FD7CFCFF3F3F5FCFF3F3D7C3FFF0F0F0F0F" ;
	    attribute INITP_03 of bram : label is "A2F48ABD2CE348AF4B0B0B0B0B0B0B0B0B0B0B0B0B0B0B0B0B0B0B0B0B0B0B0B" ;
	    attribute INITP_04 of bram : label is "A80C830AC02AAA0320C2B00AAA80C8AAAC0308AAAC03080302B00C200C0AC030" ;
	    attribute INITP_05 of bram : label is "FF7C0C0C0C00D373770CDCCDCD2CF3CEF33CF3BCCCCF33AAB00AAA80CAAB00AA" ;
	    attribute INITP_06 of bram : label is "000000000000000000000000000002DDDB02C0B02C0BF5CCFD733F5CCFBFFFFF" ;
	    attribute INITP_07 of bram : label is "C000000000000000000000000000000000000000000000000000000000000000" ;
	begin
	    bram : component RAMB16_S9_S18
	        generic map (
	            INIT_00 => X"01680160016B0B000C12016801605406CC010147093D0C20016801600160032A",
	            INIT_01 => X"C323C3220300C321037FC31103020160019A016001605412CC010147093D0C20",
	            INIT_02 => X"0F00C3360343C3350340C334C333C3320300C331037FC3260343C3250340C324",
	            INIT_03 => X"50622502025F040103100401031002CBC000032F0160016001CAC001032F02CB",
	            INIT_04 => X"03050401018601CE0236D820A83F021303001280A8C0021303200403028E0501",
	            INIT_05 => X"406101924061018E4061018A4060505E4200505C4240505A428050602680025F",
	            INIT_06 => X"020003001280A8C0020003200403027805015089250202460401031001600196",
	            INIT_07 => X"50854200508342405081428050872680024603050401018601AA0226D820A83F",
	            INIT_08 => X"414650904A0D4A104036C001030001600196408801924088018E4088018A4087",
	            INIT_09 => X"02A402130482038002A402130482034002A402130402030001600186019E01CE",
	            INIT_0A => X"02A402130483034002A402130483032002A402130483030002A40213048203C0",
	            INIT_0B => X"02A402130483038C02A402130483038802A402130483038402A4021304830380",
	            INIT_0C => X"54CB4320830102B4025F030004010160018601B201CE016002A4021304830390",
	            INIT_0D => X"019E01AA0160016054D74320830102B4025F030004070160018601A601CE0160",
	            INIT_0E => X"048203C002A402000482038002A402000482034002A402000402030001600186",
	            INIT_0F => X"0483038002A402000483034002A402000483032002A402000483030002A40200",
	            INIT_10 => X"0483039002A402000483038C02A402000483038802A402000483038402A40200",
	            INIT_11 => X"0410016055194320830102B40246030004010160018601B201AA016002A40200",
	            INIT_12 => X"0160552943808301016002C101DE0300016001860165014C1940017E01A601AA",
	            INIT_13 => X"014C195001DE02C101DE0300016001860165014C1940017E01A601AA04160160",
	            INIT_14 => X"090E090E090E1190A000C910514720044011800101600160553B432083010160",
	            INIT_15 => X"A00001478930415E8937595D490AA90F191001478930415689375955490A090E",
	            INIT_16 => X"CC01014749408B01CB40A00001470909A00001470920A0000147090A0147090D",
	            INIT_17 => X"0B2E0C0BA000016B0B2A0C04A000016B0B1C0C0EA000016B0B120C0AA000556B",
	            INIT_18 => X"0B560C09A000016B0B4D0C09A000016B0B450C08A000016B0B390C0CA000016B",
	            INIT_19 => X"0B900C04A000016B0B740C1CA000016B0B680C0CA000016B0B5F0C09A000016B",
	            INIT_1A => X"0BA10C04A000016B0B9D0C04A000016B0B990C04A000016B0B940C05A000016B",
	            INIT_1B => X"0BB50C04A000016B0BB10C04A000016B0BAD0C04A000016B0BA50C08A000016B",
	            INIT_1C => X"0BCB0C08A000016B0BC60C05A000016B0BBF0C07A000016B0BB90C06A000016B",
	            INIT_1D => X"25C04503A000016B0BFA0C05A000016B0BEA0C10A000016B0BD30C17A000016B",
	            INIT_1E => X"4703A00001DE165001DEA000450155E625804503C5030580C302C404033455DE",
	            INIT_1F => X"A000C7030740C501033455F827404703C7030740C601C302C404033455EF27C0",
	            INIT_20 => X"46224521560BA9C04926A403C426C3255605A9C04926C488A4034205C480A403",
	            INIT_21 => X"A9C04936A403C436C3355618A9C04936C488A4034218C480A403A00048244723",
	            INIT_22 => X"C622C521562BA9C04926C448A403422BC440A403A0004834473346324531561E",
	            INIT_23 => X"C632C531563BA9C04936C448A403423BC440A403A000A403C426C325C824C723",
	            INIT_24 => X"0700060606060606060607001640088A42490884A000A403C436C335C834C733",
	            INIT_25 => X"0884A000462245215659A8C04826C726C6255654A8C04826D780D63007000606",
	            INIT_26 => X"566DA8C04836D780D630070006060700060606060606060607001640088A4262",
	            INIT_27 => X"07060706070608001740094A427B0944A000463245315672A8C04836C736C635",
	            INIT_28 => X"42910944A000C826C725C622C5215686A9C04926D890D7300800070608000706",
	            INIT_29 => X"C531569CA9C04936D890D730080007060800070607060706070608001740094A",
	            INIT_2A => X"014C1960014C1970014C19800165014C1930014C19400168A000C836C735C632",
	            INIT_2B => X"0160014C1950014C19600165014C19300165014C19400168A0000160014C1950",
	            INIT_2C => X"01DE030052D42F010410A000014C19500165014C19300165014C19400147A000",
	            INIT_2D => X"454156FE460101EB03020416030CCF01570B450301DE03004305AFFC530B4503",
	            INIT_2E => X"060D030401EF05840680031B0416CF0256FE45C0A5F056FE460C01EB030356FE",
	            INIT_2F => X"016501C2430B56FA268001EB01EF05400691030001EF05400611030001EF05E1",
	            INIT_30 => X"01DE0314016801AAA000430B0160016001D6016501C2430B0160016001D20147",
	            INIT_31 => X"016801605718433C83010147195001DE032801680160570F4324830101471950",
	            INIT_32 => X"00DDA000033E02F901FA00FAA000016001605721435C83010147195001DE0344",
	            INIT_33 => X"573EC001A000033E020101FE007AA000033E020501FA007CA000033E0264018D",
	            INIT_34 => X"00000000000000000000000000000000000000000000A000573EC201573EC101",
	            INIT_35 => X"0000000000000000000000000000000000000000000000000000000000000000",
	            INIT_36 => X"0000000000000000000000000000000000000000000000000000000000000000",
	            INIT_37 => X"0000000000000000000000000000000000000000000000000000000000000000",
	            INIT_38 => X"0000000000000000000000000000000000000000000000000000000000000000",
	            INIT_39 => X"0000000000000000000000000000000000000000000000000000000000000000",
	            INIT_3A => X"0000000000000000000000000000000000000000000000000000000000000000",
	            INIT_3B => X"0000000000000000000000000000000000000000000000000000000000000000",
	            INIT_3C => X"0000000000000000000000000000000000000000000000000000000000000000",
	            INIT_3D => X"0000000000000000000000000000000000000000000000000000000000000000",
	            INIT_3E => X"0000000000000000000000000000000000000000000000000000000000000000",
	            INIT_3F => X"408C000000000000000000000000000000000000000000000000000000000000",
	            INITP_00 => X"DDDDC3F0C0C3370FFFFF77770FC3030CDC03FFFF222A2222A223FF70FC3F70FF",
	            INITP_01 => X"0F0F0F0F0F0F0F0FFFD7C3FFD7C3FFF0F0F0F0F0F0F0F0F0F0F0F0FFF4F3FFFF",
	            INITP_02 => X"0B0B0B0B71ACB2CCB7743776A8AD3FD7CFCFF3F3F5FCFF3F3D7C3FFF0F0F0F0F",
	            INITP_03 => X"A2F48ABD2CE348AF4B0B0B0B0B0B0B0B0B0B0B0B0B0B0B0B0B0B0B0B0B0B0B0B",
	            INITP_04 => X"A80C830AC02AAA0320C2B00AAA80C8AAAC0308AAAC03080302B00C200C0AC030",
	            INITP_05 => X"FF7C0C0C0C00D373770CDCCDCD2CF3CEF33CF3BCCCCF33AAB00AAA80CAAB00AA",
	            INITP_06 => X"000000000000000000000000000002DDDB02C0B02C0BF5CCFD733F5CCFBFFFFF",
	            INITP_07 => X"C000000000000000000000000000000000000000000000000000000000000000"
	        )
	        port map (
	            DIB => "0000000000000000",
	            DIPB => "00",
	            ENB => '1',
	            WEB => '0',
	            SSRB => '0',
	            CLKB => clk,
	            ADDRB => address,
	            DOB => instruction( INSTSIZE - 3 downto 0 ),
	            DOPB => instruction( INSTSIZE - 1 downto INSTSIZE - 2 ),
	            DIA => jdata( JDATASIZE - 2 downto 0 ),
	            DIPA => jdata( JDATASIZE - 1 downto JDATASIZE - 1 ),
	            ENA => juser1,
	            WEA => jwrite,
	            SSRA => '0',
	            CLKA => clk,
	            ADDRA => jaddr,
	            DOA => open,
	            DOPA => open 
	        ) ;
	end generate ;

	I16 : if PicoType = pbtI generate
		attribute INIT_00 of bram : label is "01680160016B0B000C12016801605406CC010147093D0C20016801600160032A" ;
		attribute INIT_01 of bram : label is "C323C3220300C321037FC31103020160019A016001605412CC010147093D0C20" ;
		attribute INIT_02 of bram : label is "0F00C3360343C3350340C334C333C3320300C331037FC3260343C3250340C324" ;
		attribute INIT_03 of bram : label is "50622502025F040103100401031002CBC000032F0160016001CAC001032F02CB" ;
		attribute INIT_04 of bram : label is "03050401018601CE0236D820A83F021303001280A8C0021303200403028E0501" ;
		attribute INIT_05 of bram : label is "406101924061018E4061018A4060505E4200505C4240505A428050602680025F" ;
		attribute INIT_06 of bram : label is "020003001280A8C0020003200403027805015089250202460401031001600196" ;
		attribute INIT_07 of bram : label is "50854200508342405081428050872680024603050401018601AA0226D820A83F" ;
		attribute INIT_08 of bram : label is "414650904A0D4A104036C001030001600196408801924088018E4088018A4087" ;
		attribute INIT_09 of bram : label is "02A402130482038002A402130482034002A402130402030001600186019E01CE" ;
		attribute INIT_0A of bram : label is "02A402130483034002A402130483032002A402130483030002A40213048203C0" ;
		attribute INIT_0B of bram : label is "02A402130483038C02A402130483038802A402130483038402A4021304830380" ;
		attribute INIT_0C of bram : label is "54CB4320830102B4025F030004010160018601B201CE016002A4021304830390" ;
		attribute INIT_0D of bram : label is "019E01AA0160016054D74320830102B4025F030004070160018601A601CE0160" ;
		attribute INIT_0E of bram : label is "048203C002A402000482038002A402000482034002A402000402030001600186" ;
		attribute INIT_0F of bram : label is "0483038002A402000483034002A402000483032002A402000483030002A40200" ;
	begin
	    bram : component RAMB4_S8_S16
	        generic map (
	            INIT_00 => X"01680160016B0B000C12016801605406CC010147093D0C20016801600160032A",
	            INIT_01 => X"C323C3220300C321037FC31103020160019A016001605412CC010147093D0C20",
	            INIT_02 => X"0F00C3360343C3350340C334C333C3320300C331037FC3260343C3250340C324",
	            INIT_03 => X"50622502025F040103100401031002CBC000032F0160016001CAC001032F02CB",
	            INIT_04 => X"03050401018601CE0236D820A83F021303001280A8C0021303200403028E0501",
	            INIT_05 => X"406101924061018E4061018A4060505E4200505C4240505A428050602680025F",
	            INIT_06 => X"020003001280A8C0020003200403027805015089250202460401031001600196",
	            INIT_07 => X"50854200508342405081428050872680024603050401018601AA0226D820A83F",
	            INIT_08 => X"414650904A0D4A104036C001030001600196408801924088018E4088018A4087",
	            INIT_09 => X"02A402130482038002A402130482034002A402130402030001600186019E01CE",
	            INIT_0A => X"02A402130483034002A402130483032002A402130483030002A40213048203C0",
	            INIT_0B => X"02A402130483038C02A402130483038802A402130483038402A4021304830380",
	            INIT_0C => X"54CB4320830102B4025F030004010160018601B201CE016002A4021304830390",
	            INIT_0D => X"019E01AA0160016054D74320830102B4025F030004070160018601A601CE0160",
	            INIT_0E => X"048203C002A402000482038002A402000482034002A402000402030001600186",
	            INIT_0F => X"0483038002A402000483034002A402000483032002A402000483030002A40200"
	        )
			port map (
				DIB => "0000000000000000",  
				ENB => '1', 
				WEB => '0',
				RSTB =>	'0',
				CLKB => clk,
				ADDRB => address,
				DOB => instruction( INSTSIZE - 1 downto 0 ),  
				DIA => jdata( JDATASIZE - 1 downto 0 ),   
				ENA => juser1, 
				WEA => jwrite,
				RSTA => '0',
				CLKA => clk,
				ADDRA => jaddr,
				DOA => open  
			) ; 
	end generate ;

	I20 : if PicoType = pbtS generate
		attribute INIT_00 of ram_1 : label is "3130000333333333020202220202020222020203333313003330033313003333" ;
		attribute INIT_01 of ram_1 : label is "3131313130033300300030030313003333333333131313130033300300030030" ;
		attribute INIT_02 of ram_1 : label is "3300330033003300330033003300330033003300330033333310330333333333" ;
		attribute INIT_03 of ram_1 : label is "0033003300330033003300330033003333333113300333333113300333333300" ;
		attribute INIT_04 of ram_1 : label is "3033303333033303331133303333033303311330033333330033003300330033" ;
		attribute INIT_05 of ram_1 : label is "0023002300230023130122302302303023131310031313122220223103333113" ;
		attribute INIT_06 of ram_1 : label is "0023002300230023002300230023002300230023002300230023002300230023" ;
		attribute INIT_07 of ram_1 : label is "2202331020222331023032031020223310230023002300230023002300230023" ;
		attribute INIT_08 of ram_1 : label is "2230000300202222223000030020000300022300003002000030002230000300" ;
		attribute INIT_09 of ram_1 : label is "2220003020030022300002222222000302003002230000222222200030202222" ;
		attribute INIT_0A of ram_1 : label is "3303033033032330303030330303222223000022222220003022222300002222" ;
		attribute INIT_0B of ram_1 : label is "3333133000300030003000003103130313130030313030313031023033033032" ;
		attribute INIT_0C of ram_1 : label is "3123000230002300023000233311303033311303033311303033233333333333" ;
		attribute INIT_0D of ram_1 : label is "0000000000000000000000000000000000000000000000000000000000023131" ;
		attribute INIT_0E of ram_1 : label is "0000000000000000000000000000000000000000000000000000000000000000" ;
		attribute INIT_0F of ram_1 : label is "3000000000000000000000000000000000000000000000000000000000000000" ;

		attribute INIT_00 of ram_2 : label is "52000000C0000C000C0C0CCC0C0C0C0CCC0C0C000005C00000000005C0000000" ;
		attribute INIT_01 of ram_2 : label is "54545452000000DA001A000005200000404040454545452000000DA001A00000" ;
		attribute INIT_02 of ram_2 : label is "00000000000000000000000000000000000000000000000045444C0004040404" ;
		attribute INIT_03 of ram_2 : label is "0000000000000000000000000000000000005480000000005480000000000000" ;
		attribute INIT_04 of ram_2 : label is "0100000000100000054800000000100000548000000000000000000000000000" ;
		attribute INIT_05 of ram_2 : label is "00A000A000A000A5C048CA00A00A0000A084854A108485400001AC5248005480" ;
		attribute INIT_06 of ram_2 : label is "00A000A000A000A000A000A000A000A000A000A000A000A000A000A000A000A0" ;
		attribute INIT_07 of ram_2 : label is "AC0C0524C0CCC0524A010A4524C0CC0524A000A000A000A000A000A000A000A0" ;
		attribute INIT_08 of ram_2 : label is "CC5A4CA4CAAACCCCCC5A4CA4CAA44445A4ACC5A4CA4CAA44445A4ACC5A4CA4CA" ;
		attribute INIT_09 of ram_2 : label is "00001040A445A4CC5A4DD000000001040A445A4CC5A4DD000000001040AACCCC" ;
		attribute INIT_0A of ram_2 : label is "001010010010A001010101001010ACCCC5A4DD000000001040ACCCC5A4DD0000" ;
		attribute INIT_0B of ram_2 : label is "00452000000000000000000C54A540054540000C54004A5400520A010010010A" ;
		attribute INIT_0C of ram_2 : label is "5CA0000A0000A0000A0000A00548010000548010000548010000A40000040000" ;
		attribute INIT_0D of ram_2 : label is "00000000000000000000000000000000000000000000000000000000000A5C5C" ;
		attribute INIT_0E of ram_2 : label is "0000000000000000000000000000000000000000000000000000000000000000" ;
		attribute INIT_0F of ram_2 : label is "4000000000000000000000000000000000000000000000000000000000000000" ;

		attribute INIT_00 of ram_3 : label is "0524343203111032F333333333333333333333311114C19C111BC114C19C1113" ;
		attribute INIT_01 of ram_3 : label is "0202020623411288232823425052431101010100202020623411288232823425" ;
		attribute INIT_02 of ram_3 : label is "22432243224322432243224322432243224322432243111110AA003110101010" ;
		attribute INIT_03 of ram_3 : label is "4322432243224322432243224322431111114332234111114332234111112243" ;
		attribute INIT_04 of ram_3 : label is "1912131111911141153312131111911141533223411111224322432243224322" ;
		attribute INIT_05 of ram_3 : label is "BC01BC01BC01BC05C19BB0190190191901919999919199999991091000115331" ;
		attribute INIT_06 of ram_3 : label is "BC01BC01BC01BC01BC01BC01BC01BC01BC01BC01BC01BC01BC01BC01BC01BC01" ;
		attribute INIT_07 of ram_3 : label is "077535777763435770161055555534355501BC01BC01BC01BC01BC01BC01BC01" ;
		attribute INIT_08 of ram_3 : label is "6569944244044387656994424408765699443699442440876569944369944244" ;
		attribute INIT_09 of ram_3 : label is "7778792906568876688767676666768280656887668876767666676828044387" ;
		attribute INIT_0A of ram_3 : label is "1191911911910119191919119191087656998787877778792908765699878787" ;
		attribute INIT_0B of ram_3 : label is "11366115631563156315634F655661365661343F75133F35132F401911911910" ;
		attribute INIT_0C of ram_3 : label is "7003210032100321003210011733191311733191311733191311031111131111" ;
		attribute INIT_0D of ram_3 : label is "0000000000000000000000000000000000000000000000000000000000007271" ;
		attribute INIT_0E of ram_3 : label is "0000000000000000000000000000000000000000000000000000000000000000" ;
		attribute INIT_0F of ram_3 : label is "0000000000000000000000000000000000000000000000000000000000000000" ;

		attribute INIT_00 of ram_4 : label is "6050101C0266C02C034343330372424222027106966104326660166004326662" ;
		attribute INIT_01 of ram_4 : label is "808488884008A223008C0207080401696968686505458685008C323108C12080" ;
		attribute INIT_02 of ram_4 : label is "A188A188A188A188A184A182A180A18CA188A184A100689C4901300698988888" ;
		attribute INIT_03 of ram_4 : label is "88A084A082A080A08CA088A084A000689A66D20B50068AC6C20B50068BC6A189" ;
		attribute INIT_04 of ram_4 : label is "45DCD0686447AA1662806CD0686447AA16120B40068BA6A089A088A088A088A0" ;
		attribute INIT_05 of ram_4 : label is "2006200610061006044040400420404004353500143535000009014010663206" ;
		attribute INIT_06 of ram_4 : label is "B006B006A006A006A00690069006900690067106600650065006400640063006" ;
		attribute INIT_07 of ram_4 : label is "00403F40040003EC00D5D00E8008003DC006F006E106D106C006C006B006B006" ;
		attribute INIT_08 of ram_4 : label is "333C340340002222222C240240033331C30331C380180022220C20220C280080" ;
		attribute INIT_09 of ram_4 : label is "000044740337C3336C3830000000048680225C2225C283000000004848003333" ;
		attribute INIT_0A of ram_4 : label is "6454664364460645464748643446033339C393000000004494022228C2930000" ;
		attribute INIT_0B of ram_4 : label is "6C0F8EE490E410EE00E88110FCFF0E0F4F0E010000D00F00D0D0104564364440" ;
		attribute INIT_0C of ram_4 : label is "30030F7030F70368D03FFF06625045D46613045D26602045D16A0066D6C066D4" ;
		attribute INIT_0D of ram_4 : label is "0000000000000000000000000000000000000000000000000000000000003030" ;
		attribute INIT_0E of ram_4 : label is "0000000000000000000000000000000000000000000000000000000000000000" ;
		attribute INIT_0F of ram_4 : label is "8000000000000000000000000000000000000000000000000000000000000000" ;

		attribute INIT_00 of ram_5 : label is "22F1010B0F00A1FB0635043201F635043201F120A00217D080B0280617D0800A" ;
		attribute INIT_01 of ram_5 : label is "503010706516A60F0000003819261006121E1A0E0C0A000F516E60F3000303E1" ;
		attribute INIT_02 of ram_5 : label is "433C433843344330433043304330432043204320432006EE60D061006828E8A7" ;
		attribute INIT_03 of ram_5 : label is "30403040304030402040204020402006EA007014F07066E0B014F01062E04330" ;
		attribute INIT_04 of ram_5 : label is "C0E1E0065C0E6A60090101E0065C0E6A009014601062A04030403C4038403440" ;
		attribute INIT_05 of ram_5 : label is "EB0BA40BCE0B2A0B1701007907007A7D070E7DAF070675AEEEE000741100B010" ;
		attribute INIT_06 of ram_5 : label is "540B140BD40B580B140BD40B940B450B040B4C0B8C0BF90B690BD90B580B9C0B" ;
		attribute INIT_07 of ram_5 : label is "03014803301244F030E0E0160330244E030BA50BA00B370BB80B650BF70B960B" ;
		attribute INIT_08 of ram_5 : label is "21B0683B0303654321B0683B0304321E063658068380304321B0636550683503" ;
		attribute INIT_09 of ram_5 : label is "66600AB402120665D0600060666600A240219066540600060666600A94036543" ;
		attribute INIT_0A of ram_5 : label is "0C0C05C05C0800C0C0C0C05C0C0806521C0600060666600A1406521606000606" ;
		attribute INIT_0B of ram_5 : label is "52BA0BF010F010F1D4F40B62E00ECB3E1E1B26C1B3E05CB3E04100C05C05C070" ;
		attribute INIT_0C of ram_5 : label is "E10E1EA0E5AC0E4DD0E9AA0001C170E4808C170E880F4170E48A0B00652B0027" ;
		attribute INIT_0D of ram_5 : label is "000000000000000000000000000000000000000000000000000000000000E1E1" ;
		attribute INIT_0E of ram_5 : label is "0000000000000000000000000000000000000000000000000000000000000000" ;
		attribute INIT_0F of ram_5 : label is "C000000000000000000000000000000000000000000000000000000000000000" ;

		signal data_out : std_logic_vector( 3 downto 0 ) ;
	begin
	    ram_1 : component RAMB4_S4_S4
	        generic map (
				INIT_00 => X"3130000333333333020202220202020222020203333313003330033313003333",
				INIT_01 => X"3131313130033300300030030313003333333333131313130033300300030030",
				INIT_02 => X"3300330033003300330033003300330033003300330033333310330333333333",
				INIT_03 => X"0033003300330033003300330033003333333113300333333113300333333300",
				INIT_04 => X"3033303333033303331133303333033303311330033333330033003300330033",
				INIT_05 => X"0023002300230023130122302302303023131310031313122220223103333113",
				INIT_06 => X"0023002300230023002300230023002300230023002300230023002300230023",
				INIT_07 => X"2202331020222331023032031020223310230023002300230023002300230023",
				INIT_08 => X"2230000300202222223000030020000300022300003002000030002230000300",
				INIT_09 => X"2220003020030022300002222222000302003002230000222222200030202222",
				INIT_0A => X"3303033033032330303030330303222223000022222220003022222300002222",
				INIT_0B => X"3333133000300030003000003103130313130030313030313031023033033032",
				INIT_0C => X"3123000230002300023000233311303033311303033311303033233333333333",
				INIT_0D => X"0000000000000000000000000000000000000000000000000000000000023131",
				INIT_0E => X"0000000000000000000000000000000000000000000000000000000000000000",
				INIT_0F => X"3000000000000000000000000000000000000000000000000000000000000000"
	        )
			port map (
				DIA => "0000",  
				ENA => '1', 
				WEA => '0',
				RSTA =>	'0',
				CLKA => clk,
				ADDRA => address,
				DOA => data_out,  
				DIB => jdata( JDATASIZE - 1 downto 16 ),   
				ENB => juser1, 
				WEB => jwrite,
				RSTB => '0',
				CLKB => clk,
				ADDRB => jaddr,
				DOB => open  
			) ; 
			-- loose top 2 bits
			instruction( 17 downto 16 ) <= data_out( 1 downto 0 ) ;

	    ram_2 : component RAMB4_S4_S4
	        generic map (
				INIT_00 => X"52000000C0000C000C0C0CCC0C0C0C0CCC0C0C000005C00000000005C0000000",
				INIT_01 => X"54545452000000DA001A000005200000404040454545452000000DA001A00000",
				INIT_02 => X"00000000000000000000000000000000000000000000000045444C0004040404",
				INIT_03 => X"0000000000000000000000000000000000005480000000005480000000000000",
				INIT_04 => X"0100000000100000054800000000100000548000000000000000000000000000",
				INIT_05 => X"00A000A000A000A5C048CA00A00A0000A084854A108485400001AC5248005480",
				INIT_06 => X"00A000A000A000A000A000A000A000A000A000A000A000A000A000A000A000A0",
				INIT_07 => X"AC0C0524C0CCC0524A010A4524C0CC0524A000A000A000A000A000A000A000A0",
				INIT_08 => X"CC5A4CA4CAAACCCCCC5A4CA4CAA44445A4ACC5A4CA4CAA44445A4ACC5A4CA4CA",
				INIT_09 => X"00001040A445A4CC5A4DD000000001040A445A4CC5A4DD000000001040AACCCC",
				INIT_0A => X"001010010010A001010101001010ACCCC5A4DD000000001040ACCCC5A4DD0000",
				INIT_0B => X"00452000000000000000000C54A540054540000C54004A5400520A010010010A",
				INIT_0C => X"5CA0000A0000A0000A0000A00548010000548010000548010000A40000040000",
				INIT_0D => X"00000000000000000000000000000000000000000000000000000000000A5C5C",
				INIT_0E => X"0000000000000000000000000000000000000000000000000000000000000000",
				INIT_0F => X"4000000000000000000000000000000000000000000000000000000000000000"
	        )
			port map (
				DIA => "0000",  
				ENA => '1', 
				WEA => '0',
				RSTA =>	'0',
				CLKA => clk,
				ADDRA => address,
				DOA => instruction( 15 downto 12 ),  
				DIB => jdata( 15 downto 12 ),   
				ENB => juser1, 
				WEB => jwrite,
				RSTB => '0',
				CLKB => clk,
				ADDRB => jaddr,
				DOB => open  
			) ; 

	    ram_3 : component RAMB4_S4_S4
	        generic map (
				INIT_00 => X"0524343203111032F333333333333333333333311114C19C111BC114C19C1113",
				INIT_01 => X"0202020623411288232823425052431101010100202020623411288232823425",
				INIT_02 => X"22432243224322432243224322432243224322432243111110AA003110101010",
				INIT_03 => X"4322432243224322432243224322431111114332234111114332234111112243",
				INIT_04 => X"1912131111911141153312131111911141533223411111224322432243224322",
				INIT_05 => X"BC01BC01BC01BC05C19BB0190190191901919999919199999991091000115331",
				INIT_06 => X"BC01BC01BC01BC01BC01BC01BC01BC01BC01BC01BC01BC01BC01BC01BC01BC01",
				INIT_07 => X"077535777763435770161055555534355501BC01BC01BC01BC01BC01BC01BC01",
				INIT_08 => X"6569944244044387656994424408765699443699442440876569944369944244",
				INIT_09 => X"7778792906568876688767676666768280656887668876767666676828044387",
				INIT_0A => X"1191911911910119191919119191087656998787877778792908765699878787",
				INIT_0B => X"11366115631563156315634F655661365661343F75133F35132F401911911910",
				INIT_0C => X"7003210032100321003210011733191311733191311733191311031111131111",
				INIT_0D => X"0000000000000000000000000000000000000000000000000000000000007271",
				INIT_0E => X"0000000000000000000000000000000000000000000000000000000000000000",
				INIT_0F => X"0000000000000000000000000000000000000000000000000000000000000000"
	        )
			port map (
				DIA => "0000",  
				ENA => '1', 
				WEA => '0',
				RSTA =>	'0',
				CLKA => clk,
				ADDRA => address,
				DOA => instruction( 11 downto 8 ),  
				DIB => jdata( 11 downto 8 ),   
				ENB => juser1, 
				WEB => jwrite,
				RSTB => '0',
				CLKB => clk,
				ADDRB => jaddr,
				DOB => open  
			) ; 

	    ram_4 : component RAMB4_S4_S4
	        generic map (
				INIT_00 => X"6050101C0266C02C034343330372424222027106966104326660166004326662",
				INIT_01 => X"808488884008A223008C0207080401696968686505458685008C323108C12080",
				INIT_02 => X"A188A188A188A188A184A182A180A18CA188A184A100689C4901300698988888",
				INIT_03 => X"88A084A082A080A08CA088A084A000689A66D20B50068AC6C20B50068BC6A189",
				INIT_04 => X"45DCD0686447AA1662806CD0686447AA16120B40068BA6A089A088A088A088A0",
				INIT_05 => X"2006200610061006044040400420404004353500143535000009014010663206",
				INIT_06 => X"B006B006A006A006A00690069006900690067106600650065006400640063006",
				INIT_07 => X"00403F40040003EC00D5D00E8008003DC006F006E106D106C006C006B006B006",
				INIT_08 => X"333C340340002222222C240240033331C30331C380180022220C20220C280080",
				INIT_09 => X"000044740337C3336C3830000000048680225C2225C283000000004848003333",
				INIT_0A => X"6454664364460645464748643446033339C393000000004494022228C2930000",
				INIT_0B => X"6C0F8EE490E410EE00E88110FCFF0E0F4F0E010000D00F00D0D0104564364440",
				INIT_0C => X"30030F7030F70368D03FFF06625045D46613045D26602045D16A0066D6C066D4",
				INIT_0D => X"0000000000000000000000000000000000000000000000000000000000003030",
				INIT_0E => X"0000000000000000000000000000000000000000000000000000000000000000",
				INIT_0F => X"8000000000000000000000000000000000000000000000000000000000000000"
	        )
			port map (
				DIA => "0000",  
				ENA => '1', 
				WEA => '0',
				RSTA =>	'0',
				CLKA => clk,
				ADDRA => address,
				DOA => instruction( 7 downto 4 ),  
				DIB => jdata( 7 downto 4 ),   
				ENB => juser1, 
				WEB => jwrite,
				RSTB => '0',
				CLKB => clk,
				ADDRB => jaddr,
				DOB => open  
			) ; 

	    ram_5 : component RAMB4_S4_S4
	        generic map (
				INIT_00 => X"22F1010B0F00A1FB0635043201F635043201F120A00217D080B0280617D0800A",
				INIT_01 => X"503010706516A60F0000003819261006121E1A0E0C0A000F516E60F3000303E1",
				INIT_02 => X"433C433843344330433043304330432043204320432006EE60D061006828E8A7",
				INIT_03 => X"30403040304030402040204020402006EA007014F07066E0B014F01062E04330",
				INIT_04 => X"C0E1E0065C0E6A60090101E0065C0E6A009014601062A04030403C4038403440",
				INIT_05 => X"EB0BA40BCE0B2A0B1701007907007A7D070E7DAF070675AEEEE000741100B010",
				INIT_06 => X"540B140BD40B580B140BD40B940B450B040B4C0B8C0BF90B690BD90B580B9C0B",
				INIT_07 => X"03014803301244F030E0E0160330244E030BA50BA00B370BB80B650BF70B960B",
				INIT_08 => X"21B0683B0303654321B0683B0304321E063658068380304321B0636550683503",
				INIT_09 => X"66600AB402120665D0600060666600A240219066540600060666600A94036543",
				INIT_0A => X"0C0C05C05C0800C0C0C0C05C0C0806521C0600060666600A1406521606000606",
				INIT_0B => X"52BA0BF010F010F1D4F40B62E00ECB3E1E1B26C1B3E05CB3E04100C05C05C070",
				INIT_0C => X"E10E1EA0E5AC0E4DD0E9AA0001C170E4808C170E880F4170E48A0B00652B0027",
				INIT_0D => X"000000000000000000000000000000000000000000000000000000000000E1E1",
				INIT_0E => X"0000000000000000000000000000000000000000000000000000000000000000",
				INIT_0F => X"C000000000000000000000000000000000000000000000000000000000000000"
	        )
			port map (
				DIA => "0000",  
				ENA => '1', 
				WEA => '0',
				RSTA =>	'0',
				CLKA => clk,
				ADDRA => address,
				DOA => instruction( 3 downto 0 ),  
				DIB => jdata( 3 downto 0 ),   
				ENB => juser1, 
				WEB => jwrite,
				RSTB => '0',
				CLKB => clk,
				ADDRB => jaddr,
				DOB => open  
			) ; 
	end generate ;

	jdata <= ( others => '0' ) ;
	jaddr <= ( others => '0' ) ;
	juser1 <= '0' ;
	jwrite <= '0' ;
end architecture mix ;
