--------------------------------------------------------------------------------
LIBRARY ieee;
USE ieee.std_logic_1164.ALL;
--------------------------------------------------------------------------------
ENTITY TB_Two_Wire_Controler IS
END TB_Two_Wire_Controler;
--------------------------------------------------------------------------------
ARCHITECTURE behavior OF TB_Two_Wire_Controler IS 
--------------------------------------------------------------------------------
  COMPONENT Two_Wire_Controler
    GENERIC(data_port_ID:   STD_LOGIC_VECTOR(7 DOWNTO 0):= "00000001";  -- data register port ID
            addr_port_ID:   STD_LOGIC_VECTOR(7 DOWNTO 0):= "00000010";  -- address register upper byte port ID
            stat_port_ID:   STD_LOGIC_VECTOR(7 DOWNTO 0):= "00000100";  -- status/control register port ID
            dadr_port_ID:   STD_LOGIC_VECTOR(7 DOWNTO 0):= "00000100";  -- device address register port ID
            clk_div:      INTEGER:= 2000);                              -- divider of system clock for SCLK (max 100 kHz for SFP)
                                                                        -- f(clk) = 100 MHz AND clk_div = 1000 => f(SCLK) = 50 kHz
    PORT(
            clk:            IN  STD_LOGIC;                              -- system clock

            -- PicoBlaze interface
            Data_in:        IN  STD_LOGIC_VECTOR(7 DOWNTO 0);           -- Data IN (PicoBlaze OUT)
            Data_out:       OUT STD_LOGIC_VECTOR(7 DOWNTO 0):= X"00";   -- Data reg OUT (to PicoBlaze input MUX)
            Address:        IN  STD_LOGIC_VECTOR(7 DOWNTO 0);           -- PicoBlaze Port ID
            write_strobe:   IN  STD_LOGIC;                              -- PicoBlaze write strobe

            -- MDIO physical interface
            SCL:            OUT STD_LOGIC;
            SDA:            INOUT STD_LOGIC := '1');
  END COMPONENT;
  ------------------------------------------------------------------------------
  --Inputs
  SIGNAL clk        : STD_LOGIC := '0';
  SIGNAL Data_in    : STD_LOGIC_VECTOR(7 DOWNTO 0) := (OTHERS => '0');
  SIGNAL Addr       : STD_LOGIC_VECTOR(7 DOWNTO 0) := (OTHERS => '0');
  SIGNAL wr         : STD_LOGIC := '0';

  --BiDirs
  SIGNAL SDA        : STD_LOGIC;

  --Outputs
  SIGNAL Data_out   : STD_LOGIC_VECTOR(7 DOWNTO 0);
  SIGNAL SCL        : STD_LOGIC;

  -- Clock period definitions
  CONSTANT clk_period   : TIME := 10 ns;
 
--------------------------------------------------------------------------------
BEGIN
--------------------------------------------------------------------------------
 
	-- Instantiate the Unit Under Test (UUT)
  uut: Two_Wire_Controler
    GENERIC MAP(
      data_port_ID  => X"01",
      addr_port_ID  => X"02",
      stat_port_ID  => X"03",
      dadr_port_ID  => X"04",
      clk_div       => 5)
    PORT MAP(
      clk           => clk,
      Data_in       => Data_in,
      Data_out      => Data_out,
      Address       => Addr,
      write_strobe  => wr,
      SCL           => SCL,
      SDA           => SDA);

  ------------------------------------------------------------------------------

  clk_process : PROCESS BEGIN
    clk <= '0'; WAIT FOR clk_period/2;
    clk <= '1'; WAIT FOR clk_period/2;
  END PROCESS;
 
  ------------------------------------------------------------------------------


  -- Stimulus process
  ------------------------------------------------------------------------------
  stim_proc: PROCESS BEGIN
    SDA     <= 'H';
    Data_in <= X"00";
    Addr    <= X"00";
    wr      <= '0';
    ----------------------------------------------------------------------------
    WAIT FOR clk_period*10;
    ----------------------------------------------------------------------------
    -- set device address
    Addr <= X"04"; Data_in <= X"55"; WAIT FOR clk_period; wr <= '1'; WAIT FOR clk_period; wr <= '0'; WAIT FOR clk_period*2;
    ----------------------------------------------------------------------------
    -- set register address
    Addr <= X"02"; Data_in <= X"0A"; WAIT FOR clk_period; wr <= '1'; WAIT FOR clk_period; wr <= '0'; WAIT FOR clk_period*2;
    ----------------------------------------------------------------------------
    -- initialize read operation
    Addr <= X"03"; Data_in <= X"80"; WAIT FOR clk_period; wr <= '1'; WAIT FOR clk_period; wr <= '0'; WAIT FOR clk_period*2;

    ----------------------------------------------------------------------------
    WAIT;
    ----------------------------------------------------------------------------
  END PROCESS;

--------------------------------------------------------------------------------
END;
--------------------------------------------------------------------------------
