----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;
library UNISIM;
use UNISIM.Vcomponents.ALL;
----------------------------------------------------------------------------------
ENTITY Tx_data_gen IS PORT(
  clk               : IN  STD_LOGIC;
  Tx_Gen_Data_out   : OUT STD_LOGIC_VECTOR(15 DOWNTO 0);
  Tx_Gen_DV         : OUT STD_LOGIC;
  Tx_Gen_First      : OUT STD_LOGIC;
  Tx_Gen_Last       : OUT STD_LOGIC;
  Tx_Gen_Ready      : IN  STD_LOGIC);
END Tx_data_gen;
----------------------------------------------------------------------------------
ARCHITECTURE Behavioral OF Tx_data_gen IS
----------------------------------------------------------------------------------
  SIGNAL Tx_start:      STD_LOGIC := '0';
  SIGNAL Tx_state:      INTEGER RANGE 0 TO 255 := 0;
  SIGNAL cnt_100ms:     STD_LOGIC_VECTOR(23 DOWNTO 0) := (OTHERS => '0');

----------------------------------------------------------------------------------
BEGIN
----------------------------------------------------------------------------------

  -- start Tx every 100 ms
  PROCESS(clk) BEGIN
    IF rising_edge(clk) THEN
      Tx_start  <= '0';
      IF cnt_100ms = X"BEBC20" THEN
        cnt_100ms <= (OTHERS => '0');
        Tx_start  <= '1';
      ELSE
        cnt_100ms <= cnt_100ms + 1;
      END IF;
    END IF;
  END PROCESS;

  --------------------------------------------------------------------------------

  PROCESS(clk) BEGIN
    IF rising_edge(clk) THEN

      ----------------------------------------------------------------------------
      -- default assignment
      Tx_state      <= Tx_state + 1;
      Tx_Gen_DV     <= '1';
      Tx_Gen_First  <= '0';
      Tx_Gen_Last   <= '0';

      ----------------------------------------------------------------------------
      -- Tx state machine (data generator)
      CASE Tx_state IS

        WHEN   0 => Tx_Gen_Data_out <= X"3020"; Tx_Gen_First <= '1';
        WHEN   1 => Tx_Gen_Data_out <= X"3120";
        WHEN   2 => Tx_Gen_Data_out <= X"3220";
        WHEN   3 => Tx_Gen_Data_out <= X"3320";
        WHEN   4 => Tx_Gen_Data_out <= X"3420";
        WHEN   5 => Tx_Gen_Data_out <= X"3520";
        WHEN   6 => Tx_Gen_Data_out <= X"3620";
        WHEN   7 => Tx_Gen_Data_out <= X"3720";
        WHEN   8 => Tx_Gen_Data_out <= X"3820";
        WHEN   9 => Tx_Gen_Data_out <= X"3920";
        WHEN  10 => Tx_Gen_Data_out <= X"3A20";
        WHEN  11 => Tx_Gen_Data_out <= X"3B20";
        WHEN  12 => Tx_Gen_Data_out <= X"3C20";
        WHEN  13 => Tx_Gen_Data_out <= X"3D20";
        WHEN  14 => Tx_Gen_Data_out <= X"3E20";
        WHEN  15 => Tx_Gen_Data_out <= X"3F20";
        WHEN  16 => Tx_Gen_Data_out <= X"4020";
        WHEN  17 => Tx_Gen_Data_out <= X"4120";
        WHEN  18 => Tx_Gen_Data_out <= X"4220";
        WHEN  19 => Tx_Gen_Data_out <= X"4320";
        WHEN  20 => Tx_Gen_Data_out <= X"4420";
        WHEN  21 => Tx_Gen_Data_out <= X"4520";
        WHEN  22 => Tx_Gen_Data_out <= X"4620";
        WHEN  23 => Tx_Gen_Data_out <= X"4720";
        WHEN  24 => Tx_Gen_Data_out <= X"4820";
        WHEN  25 => Tx_Gen_Data_out <= X"4920";
        WHEN  26 => Tx_Gen_Data_out <= X"4A20";
        WHEN  27 => Tx_Gen_Data_out <= X"4B20";
        WHEN  28 => Tx_Gen_Data_out <= X"4C20";
        WHEN  29 => Tx_Gen_Data_out <= X"4D20";
        WHEN  30 => Tx_Gen_Data_out <= X"4E20";
        WHEN  31 => Tx_Gen_Data_out <= X"4F20";
        WHEN  32 => Tx_Gen_Data_out <= X"5020";
        WHEN  33 => Tx_Gen_Data_out <= X"5120";
        WHEN  34 => Tx_Gen_Data_out <= X"5220";
        WHEN  35 => Tx_Gen_Data_out <= X"5320";
        WHEN  36 => Tx_Gen_Data_out <= X"5420";
        WHEN  37 => Tx_Gen_Data_out <= X"5520";
        WHEN  38 => Tx_Gen_Data_out <= X"5620";
        WHEN  39 => Tx_Gen_Data_out <= X"5720";
        WHEN  40 => Tx_Gen_Data_out <= X"5820";
        WHEN  41 => Tx_Gen_Data_out <= X"5920";
        WHEN  42 => Tx_Gen_Data_out <= X"5A20";
        WHEN  43 => Tx_Gen_Data_out <= X"5B20";
        WHEN  44 => Tx_Gen_Data_out <= X"5C20";
        WHEN  45 => Tx_Gen_Data_out <= X"5D20";
        WHEN  46 => Tx_Gen_Data_out <= X"5E20";
        WHEN  47 => Tx_Gen_Data_out <= X"5F20"; 
        WHEN  48 => Tx_Gen_Data_out <= X"6020";
        WHEN  49 => Tx_Gen_Data_out <= X"6120";
        WHEN  50 => Tx_Gen_Data_out <= X"6220";
        WHEN  51 => Tx_Gen_Data_out <= X"6320";
        WHEN  52 => Tx_Gen_Data_out <= X"6420";
        WHEN  53 => Tx_Gen_Data_out <= X"6520";
        WHEN  54 => Tx_Gen_Data_out <= X"6620";
        WHEN  55 => Tx_Gen_Data_out <= X"6720";
        WHEN  56 => Tx_Gen_Data_out <= X"6820";
        WHEN  57 => Tx_Gen_Data_out <= X"6920";
        WHEN  58 => Tx_Gen_Data_out <= X"6A20";
        WHEN  59 => Tx_Gen_Data_out <= X"6B20";
        WHEN  60 => Tx_Gen_Data_out <= X"6C20";
        WHEN  61 => Tx_Gen_Data_out <= X"6D20";
        WHEN  62 => Tx_Gen_Data_out <= X"6E20";
        WHEN  63 => Tx_Gen_Data_out <= X"6F20"; Tx_Gen_Last <= '1';

        -- IDLE sequence ---------------------------------------------------------
        WHEN OTHERS => Tx_Gen_DV <= '0';
                      IF Tx_start = '1' AND Tx_Gen_Ready = '1' THEN Tx_state <= 0;
                                                               ELSE Tx_state <= 255; END IF;
        --------------------------------------------------------------------------
      END CASE;
    END IF;
  END PROCESS;

----------------------------------------------------------------------------------
END Behavioral;
----------------------------------------------------------------------------------
